@@ -4,9 +4,7 @@
 #undef putw
 
 int
-putw(w, fp)
-     int w;
-     FILE *fp;
+putw( int w, FILE *fp )
 {
   _IO_size_t written;
   CHECK_FILE(fp, EOF);
@@ -22,6 +20,10 @@ putw(w, fp)
 
 #ifndef _LIBPTHREAD
 #ifdef __ELF__
+#ifdef __GNUC__
+int putw( int, FILE * ) __attribute__ ((weak));
+#else
 #pragma weak putw
-#endif
-#endif
+#endif   /* __GNUC__ */
+#endif   /* __ELF__ */
+#endif   /* _LIBPTHREAD */
