@@ -27,11 +27,10 @@ Cambridge, MA 02139, USA.  */
 
 char *__progname = (char *) "";
 
+static void __init_misc ( int, char **, char ** );
+
 static void
-__init_misc (argc, argv, envp)
-     int argc;
-     char **argv;
-     char **envp;
+__init_misc ( int argc, char **argv, char **envp )
 {
   if (argv && argv[0])
     {
@@ -44,5 +43,9 @@ __init_misc (argc, argv, envp)
 }
 
 #ifdef HAVE_GNU_LD
+#if 0
 text_set_element (__libc_subinit, __init_misc);
+#else
+data_set_element (__libc_subinit, __init_misc);
+#endif
 #endif
