@@ -1,5 +1,5 @@
 /* Read decimal floating point numbers.
-Copyright (C) 1995 Free Software Foundation, Inc.
+Copyright (C) 1995, 1996 Free Software Foundation, Inc.
 Contributed by Ulrich Drepper.
 
 This file is part of the GNU C Library.
@@ -106,7 +106,7 @@ static const mp_limb _tens_in_limb[MAX_DIG_PER_LIMB + 1] =
 #define	MPNSIZE		(howmany (MAX_EXP + 2 * MANT_DIG, BITS_PER_MP_LIMB) \
 			 + 2)
 /* Declare an mpn integer variable that big.  */
-#define	MPN_VAR(name)	mp_limb name[MPNSIZE]; mp_size_t name##size
+#define	MPN_VAR(name)	mp_limb name[MPNSIZE] = {0}; mp_size_t name##size
 /* Copy an mpn integer value.  */
 #define MPN_ASSIGN(dst, src) \
 	memcpy (dst, src, (dst##size = src##size) * sizeof (mp_limb))
@@ -406,7 +406,7 @@ INTERNAL (STRTOF) (nptr, endptr, group)
 
   /* If no other digit but a '0' is found the result is 0.0.
      Return current read pointer.  */
-  if (!isdigit (c) && c != decimal)
+  if (!isdigit (c) && c != decimal && tolower (c) != 'e')
     {
       tp = correctly_grouped_prefix (start_of_digits, cp, thousands, grouping);
       /* If TP is at the start of the digits, there was no correctly
@@ -483,6 +483,8 @@ INTERNAL (STRTOF) (nptr, endptr, group)
 	++dig_no;
       }
 
+  assert (lead_zero != -1);
+
   /* Remember start of exponent (if any).  */
   expp = cp;
 
@@ -502,11 +504,18 @@ INTERNAL (STRTOF) (nptr, endptr, group)
 
       if (isdigit (c))
 	{
+	  int exp_limit;
+
+	  /* Get the exponent limit. */
+	  exp_limit = exp_negative ?
+		-MIN_10_EXP + MANT_DIG - int_no :
+		MAX_10_EXP - int_no + lead_zero;
+
 	  do
 	    {
-	      if ((!exp_negative && exponent * 10 + int_no > MAX_10_EXP)
-		  || (exp_negative
-		      && exponent * 10 + int_no > -MIN_10_EXP + MANT_DIG))
+	      exponent *= 10;
+	      
+	      if (exponent > exp_limit)
 		/* The exponent is too large/small to represent a valid
 		   number.  */
 		{
@@ -526,17 +535,16 @@ INTERNAL (STRTOF) (nptr, endptr, group)
 		  /* NOTREACHED */
 		}
 
-	      exponent *= 10;
 	      exponent += c - '0';
 	      c = *++cp;
 	    }
 	  while (isdigit (c));
+
+	  if (exp_negative)
+	    exponent = -exponent;
 	}
       else
 	cp = expp;
-
-      if (exp_negative)
-	exponent = -exponent;
     }
 
   /* We don't want to have to work with trailing zeroes after the radix.  */
@@ -559,6 +567,15 @@ INTERNAL (STRTOF) (nptr, endptr, group)
   if (dig_no == 0)
     return 0.0;
 
+  if (lead_zero)
+  {
+    /* Find the decimal point */
+    while (*startp != decimal) startp++;
+    startp += lead_zero + 1;
+    exponent -= lead_zero;
+    dig_no -= lead_zero;
+  }
+
   /* Now we have the number of digits in total and the integer digits as well
      as the exponent and its sign.  We can decide whether the read digits are
      really integer digits or belong to the fractional part; i.e. we normalize
@@ -576,7 +593,7 @@ INTERNAL (STRTOF) (nptr, endptr, group)
       return negative ? -FLOAT_HUGE_VAL : FLOAT_HUGE_VAL;
     }
 
-  if (exponent - MAX(0, lead_zero) < MIN_10_EXP - (DIG + 1))
+  if (exponent < MIN_10_EXP - (DIG + 1))
     {
       errno = ERANGE;
       return 0.0;
@@ -1109,9 +1126,13 @@ __STRTOF
 #else
 STRTOF
 #endif
+#ifdef __STDC__
+   ( const char *nptr, char **endptr)
+#else
 	(nptr, endptr)
      const char *nptr;
      char **endptr;
+#endif
 {
   return INTERNAL (STRTOF) (nptr, endptr, 0);
 }
