@@ -1,9 +1,14 @@
 #include <syscall.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <gnu-stabs.h>
 
+#ifdef __SVR4_I386_ABI_L1__
+#define prev_lstat	__prev_lstat
+#else
 static inline
 _syscall2(int,prev_lstat,const char *,path, struct stat *,statbuf)
+#endif
 
 int
 _lxstat(int version, const char * path, struct stat * statbuf)
@@ -17,3 +22,5 @@ _lxstat(int version, const char * path, struct stat * statbuf)
     return -1;
   }
 }
+
+elf_alias (_lxstat, __lxstat);
